require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_addomaindefaultpasswordpolicy',
  dscmeta_resource_friendly_name: 'ADDomainDefaultPasswordPolicy',
  dscmeta_resource_name: 'MSFT_ADDomainDefaultPasswordPolicy',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC ADDomainDefaultPasswordPolicy resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_minpasswordlength: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Minimum number of characters that a password must contain.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_complexityenabled: {
      type: 'Optional[Boolean]',
      desc: 'Whether password complexity is enabled for the default password policy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_maxpasswordage: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Maximum length of time that you can have the same password (minutes).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_reversibleencryptionenabled: {
      type: 'Optional[Boolean]',
      desc: 'Whether the directory must store passwords using reversible encryption.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_lockoutobservationwindow: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Maximum time between two unsuccessful login attempts before the counter is reset to 0 (minutes).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to access the domain.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_passwordhistorycount: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Number of previous passwords to remember.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'Active Directory domain controller to enact the change upon.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_lockoutthreshold: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Number of unsuccessful login attempts that are permitted before an account is locked out.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_lockoutduration: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Length of time that an account is locked after the number of failed login attempts (minutes).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_domainname: {
      type: 'String',
      desc: 'Name of the domain to which the password policy will be applied.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_minpasswordage: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Minimum length of time that you can have the same password (minutes).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
