require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_addomaintrust',
  dscmeta_resource_friendly_name: 'ADDomainTrust',
  dscmeta_resource_name: 'MSFT_ADDomainTrust',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC ADDomainTrust resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_trusttype: {
      type: "Enum['External', 'Forest']",
      desc: "Specifies the type of trust. The value 'External' means the context Domain, while the value 'Forest' means the context 'Forest'.",

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_trustdirection: {
      type: "Enum['Bidirectional', 'Inbound', 'Outbound']",
      desc: 'Specifies the direction of the trust.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: "Specifies whether the computer account is present or absent. Default value is 'Present'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowtrustrecreation: {
      type: 'Optional[Boolean]',
      desc: 'Specifies if the is allowed to be recreated if required. Default value is $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_sourcedomainname: {
      type: 'String',
      desc: 'Specifies the name of the Active Directory domain that is requesting the trust.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_targetdomainname: {
      type: 'String',
      desc: 'Specifies the name of the Active Directory domain that is being trusted.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_targetcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the credentials to authenticate to the target domain.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
  },
)
