require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adobjectenabledstate',
  dscmeta_resource_friendly_name: 'ADObjectEnabledState',
  dscmeta_resource_name: 'MSFT_ADObjectEnabledState',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC ADObjectEnabledState resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_enabled: {
      type: 'Boolean',
      desc: 'Specifies the value of the Enabled property.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the user account credentials to use to perform the task.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_identity: {
      type: 'String',
      desc: "Specifies the identity of an object that has the object class specified in the parameter ObjectClass. When ObjectClass is set to 'Computer' then this property can be set to either distinguished name, GUID (objectGUID), security identifier (objectSid), or security Accounts Manager account name (sAMAccountName).",
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'Specifies the Active Directory Domain Services instance to connect to perform the task.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_objectclass: {
      type: "Enum['Computer']",
      desc: 'Specifies the object class.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
