require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_adorganizationalunit',
  dscmeta_resource_friendly_name: 'ADOrganizationalUnit',
  dscmeta_resource_name: 'MSFT_ADOrganizationalUnit',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC ADOrganizationalUnit resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_protectedfromaccidentaldeletion: {
      type: 'Optional[Boolean]',
      desc: 'Specifies if the Organization Unit (OU) container should be protected from deletion. Default value is $true.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: "Specifies whether the Organization Unit (OU) is present or absent. Default value is 'Present'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: "The description of the Organization Unit (OU). Default value is empty ('') description.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'The name of Organization Unit (OU).',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'The credential to be used to perform the operation on Active Directory.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_path: {
      type: 'String',
      desc: 'Specifies the X.500 path of the Organization Unit (OU) or container where the new object is created.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_restorefromrecyclebin: {
      type: 'Optional[Boolean]',
      desc: 'Try to restore the Organization Unit (OU) from the recycle bin before creating a new one.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
