require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_aduser',
  dscmeta_resource_friendly_name: 'ADUser',
  dscmeta_resource_name: 'MSFT_ADUser',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC ADUser resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_company: {
      type: 'Optional[String]',
      desc: "Specifies the user's company (ldapDisplayName 'company').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homepage: {
      type: 'Optional[String]',
      desc: "Specifies the URL of the home page of the object (ldapDisplayName 'wWWHomePage').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ipphone: {
      type: 'Optional[String]',
      desc: "Specifies the user's IP telephony phone number (ldapDisplayName 'ipPhone').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: "Specifies a description of the object (ldapDisplayName 'description').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_profilepath: {
      type: 'Optional[String]',
      desc: "Specifies a path to the user's profile (ldapDisplayName 'profilePath').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_distinguishedname: {
      type: 'Optional[String]',
      desc: 'Returns the X.500 path of the object.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_restorefromrecyclebin: {
      type: 'Optional[Boolean]',
      desc: 'Try to restore the user object from the recycle bin before creating a new one.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_office: {
      type: 'Optional[String]',
      desc: "Specifies the location of the user's office or place of business (ldapDisplayName 'physicalDeliveryOfficeName').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_displayname: {
      type: 'Optional[String]',
      desc: "Specifies the display name of the object (ldapDisplayName 'displayName').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enabled: {
      type: 'Optional[Boolean]',
      desc: 'Specifies if the account is enabled. Default value is $true.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_password: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies a new password value for the account.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'Specifies the Active Directory Domain Services instance to use to perform the task.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_pager: {
      type: 'Optional[String]',
      desc: "Specifies the user's pager number (ldapDisplayName 'pager').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'Optional[String]',
      desc: 'Specifies the X.500 path of the Organizational Unit (OU) or container where the new object is created.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_organization: {
      type: 'Optional[String]',
      desc: "Specifies the user's organization. This parameter sets the Organization property of a user object. The LDAP display name (ldapDisplayName) of this property is 'o'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_thumbnailphoto: {
      type: 'Optional[String]',
      desc: "Specifies the thumbnail photo to be used for the user object. Can be set either to a path pointing to a .jpg-file, or to a Base64-encoded jpeg image. If set to an empty string ('') the current thumbnail photo will be removed. The property ThumbnailPhoto will always return the image as a Base64-encoded string even if the configuration specified a file path.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_emailaddress: {
      type: 'Optional[String]',
      desc: "Specifies the user's e-mail address (ldapDisplayName 'mail').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_fax: {
      type: 'Optional[String]',
      desc: "Specifies the user's fax phone number (ldapDisplayName 'facsimileTelephoneNumber').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_accountnotdelegated: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the security context of the user is delegated to a service.  When this parameter is set to true, the security context of the account is not delegated to a service even when the service account is set as trusted for Kerberos delegation. This parameter sets the AccountNotDelegated property for an Active Directory account. This parameter also sets the ADS_UF_NOT_DELEGATED flag of the Active Directory User Account Control (UAC) attribute.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serviceprincipalnames: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies the service principal names for the user account.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_userprincipalname: {
      type: 'Optional[String]',
      desc: "Specifies the User Principal Name (UPN) assigned to the user account (ldapDisplayName 'userPrincipalName').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homedirectory: {
      type: 'Optional[String]',
      desc: "Specifies a user's home directory path (ldapDisplayName 'homeDirectory').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_country: {
      type: 'Optional[String]',
      desc: "Specifies the country or region code for the user's language of choice (ldapDisplayName 'c').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logonscript: {
      type: 'Optional[String]',
      desc: "Specifies a path to the user's log on script (ldapDisplayName 'scriptPath').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_othername: {
      type: 'Optional[String]',
      desc: "Specifies a name in addition to a user's given name and surname, such as the user's middle name. This parameter sets the OtherName property of a user object. The LDAP display name (ldapDisplayName) of this property is 'middleName'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_thumbnailphotohash: {
      type: 'Optional[String]',
      desc: 'Return the MD5 hash of the current thumbnail photo, or $null if no thumbnail photo exist.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_state: {
      type: 'Optional[String]',
      desc: "Specifies the user's or Organizational Unit's state or province (ldapDisplayName 'st').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_city: {
      type: 'Optional[String]',
      desc: "Specifies the user's town or city (ldapDisplayName 'l').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_smartcardlogonrequired: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether a smart card is required to logon. This parameter sets the SmartCardLoginRequired property for a user object. This parameter also sets the ADS_UF_SMARTCARD_REQUIRED flag of the Active Directory User Account Control attribute.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the user account credentials to use to perform this task.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_cannotchangepassword: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether the account password can be changed.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_commonname: {
      type: 'Optional[String]',
      desc: "Specifies the common name assigned to the user account (ldapDisplayName 'cn'). If not specified the default value will be the same value provided in parameter UserName.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_department: {
      type: 'Optional[String]',
      desc: "Specifies the user's department (ldapDisplayName 'department').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_trustedfordelegation: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether an account is trusted for Kerberos delegation. Default value is $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_passwordauthentication: {
      type: "Optional[Enum['Default', 'Negotiate']]",
      desc: "Specifies the authentication context type used when testing passwords. Default value is 'Default'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_employeenumber: {
      type: 'Optional[String]',
      desc: "Specifies the user's employee number (ldapDisplayName 'employeeNumber').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_proxyaddresses: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies the proxy addresses for the user account.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_passwordneverexpires: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether the password of an account can expire.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_initials: {
      type: 'Optional[String]',
      desc: "Specifies the initials that represent part of a user's name (ldapDisplayName 'initials').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: "Specifies whether the user account should be present or absent. Default value is 'Present'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_division: {
      type: 'Optional[String]',
      desc: "Specifies the user's division (ldapDisplayName 'division').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_jobtitle: {
      type: 'Optional[String]',
      desc: "Specifies the user's title (ldapDisplayName 'title').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_givenname: {
      type: 'Optional[String]',
      desc: "Specifies the user's given name (ldapDisplayName 'givenName').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_employeeid: {
      type: 'Optional[String]',
      desc: "Specifies the user's employee ID (ldapDisplayName 'employeeID').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_mobilephone: {
      type: 'Optional[String]',
      desc: "Specifies the user's mobile phone number (ldapDisplayName 'mobile').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_manager: {
      type: 'Optional[String]',
      desc: "Specifies the user's manager specified as a Distinguished Name (ldapDisplayName 'manager').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_homephone: {
      type: 'Optional[String]',
      desc: "Specifies the user's home telephone number (ldapDisplayName 'homePhone').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_streetaddress: {
      type: 'Optional[String]',
      desc: "Specifies the user's street address (ldapDisplayName 'streetAddress').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_logonworkstations: {
      type: 'Optional[String]',
      desc: 'Specifies the computers that the user can access. To specify more than one computer, create a single comma-separated list. You can identify a computer by using the Security Account Manager (SAM) account name (sAMAccountName) or the DNS host name of the computer. The SAM account name is the same as the NetBIOS name of the computer. The LDAP display name (ldapDisplayName) for this property is userWorkStations.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_username: {
      type: 'String',
      desc: "Specifies the Security Account Manager (SAM) account name of the user (ldapDisplayName 'sAMAccountName').",
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowreversiblepasswordencryption: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether reversible password encryption is allowed for the account. This parameter sets the AllowReversiblePasswordEncryption property of the account. This parameter also sets the ADS_UF_ENCRYPTED_TEXT_PASSWORD_ALLOWED flag of the Active Directory User Account Control (UAC) attribute.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_changepasswordatlogon: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether the account password must be changed during the next logon attempt. This will only be enabled when the user is initially created. This parameter cannot be set to $true if the parameter PasswordNeverExpires is also set to $true.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_homedrive: {
      type: 'Optional[String]',
      desc: "Specifies a drive that is associated with the UNC path defined by the HomeDirectory property (ldapDisplayName 'homeDrive').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passwordneverresets: {
      type: 'Optional[Boolean]',
      desc: "Specifies whether existing user's password should be reset. Default value is $false.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_surname: {
      type: 'Optional[String]',
      desc: "Specifies the user's last name or surname (ldapDisplayName 'sn').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_notes: {
      type: 'Optional[String]',
      desc: "Specifies the notes attached to the user's accoutn (ldapDisplayName 'info').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domainname: {
      type: 'String',
      desc: 'Name of the domain where the user account is located (only used if password is managed).',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_pobox: {
      type: 'Optional[String]',
      desc: "Specifies the user's post office box number (ldapDisplayName 'postOfficeBox').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passwordnotrequired: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether the account requires a password. A password is not required for a new account. This parameter sets the PasswordNotRequired property of an account object.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_postalcode: {
      type: 'Optional[String]',
      desc: "Specifies the user's postal code or zip code (ldapDisplayName 'postalCode').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_officephone: {
      type: 'Optional[String]',
      desc: "Specifies the user's office telephone number (ldapDisplayName 'telephoneNumber').",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_compoundidentitysupported: {
      type: 'Optional[Boolean]',
      desc: "Specifies whether an account supports Kerberos service tickets which includes the authorization data for the user's device. This value sets the compound identity supported flag of the Active Directory msDS-SupportedEncryptionTypes attribute.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
