require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_waitforaddomain',
  dscmeta_resource_friendly_name: 'WaitForADDomain',
  dscmeta_resource_name: 'MSFT_WaitForADDomain',
  dscmeta_module_name: 'ActiveDirectoryDsc',
  dscmeta_module_version: '4.2.0.0',
  docs: 'The DSC WaitForADDomain resource type.
         Automatically generated from version 4.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_waittimeout: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Specifies the timeout in seconds that the resource will wait for the domain to be accessible. Default value is 300 seconds.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_isavailable: {
      type: 'Optional[Boolean]',
      desc: 'Returns a value indicating if a domain controller was found.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_waitforvalidcredentials: {
      type: 'Optional[Boolean]',
      desc: 'Specifies that the resource will not throw an error if authentication fails using the provided credentials and continue wait for the timeout. This can be used if the credentials are known to eventually exist but there are a potential timing issue before they are accessible.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_domainname: {
      type: 'String',
      desc: 'Specifies the fully qualified domain name to wait for.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies the credentials that are used when accessing the domain, unless the built-in PsDscRunAsCredential is used.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_sitename: {
      type: 'Optional[String]',
      desc: 'Specifies the site in the domain where to look for a domain controller.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_restartcount: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Specifies the number of times the node will be reboot in an effort to connect to the domain.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
